#coding=utf8
import subprocess
import tempfile

from django.db import models
from django.db.models.signals import post_save, post_delete, pre_save, pre_delete
from django.template.defaultfilters import slugify
from OpenDirectory.models import Users, Service, Computer
from OpenDirAdmin.settings import PERSSL_PATH, OPENDIRADMIN_DOMAIN, OPENDIRADMIN_ORGANIZATION, PERSSL_SERVICE_OU, PERSSL_SERVICE_L, PERSSL_SERVICE_ST, PERSSL_SERVICE_C
from OpenDirAdmin.settings import PERSSL_COMPUTER_OU, PERSSL_COMPUTER_L, PERSSL_COMPUTER_ST, PERSSL_COMPUTER_C, OPENDIRADMIN_SERVICE_PREFIX, OPENDIRADMIN_USER_PREFIX, OPENDIRADMIN_COMPUTER_PREFIX

def clean_name(name):
    return name.lower()
    return slugify(name.lower())
    if name.find(OPENDIRADMIN_DOMAIN) > -1:
        name = name[0:name.find(OPENDIRADMIN_DOMAIN) - 1]
    return slugify(name)


def check_output(*popenargs, **kwargs):
    r"""Run command with arguments and return its output as a byte string.

    Backported from Python 2.7 as it's implemented as pure python on stdlib.

    >>> check_output(['/usr/bin/python', '--version'])
    Python 2.6.2
    """
    process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
    output, unused_err = process.communicate()
    retcode = process.poll()
    if retcode:
        cmd = kwargs.get("args")
        if cmd is None:
            cmd = popenargs[0]
        error = subprocess.CalledProcessError(retcode, cmd)
        error.output = output
        raise error
    return output

def perssl_command(command):
    command_line = u'cd {0} && ./manage.py {1} --settings settings'.format(PERSSL_PATH, command).encode('utf-8')
    print command_line
    return check_output(command_line, shell=True, cwd=PERSSL_PATH)


def perssl_execute(command, principal, query):
    command_line = u'cd {0} && ./manage.py principal --settings settings {1} "{2}" --batch {3}'.format(PERSSL_PATH, command, principal, query).encode('utf-8')
    print command_line
    return subprocess.check_call(command_line, shell=True, cwd=PERSSL_PATH)

def create_perssl_user(sender, instance, created, *args, **kwargs):
    values = [
            ('role', 'user'),
            ('commonName', instance.cn),
            ('organization', instance.applecompany),
            ('organizationalUnit', instance.departmentnumber),
            ('locality', instance.l),
            ('stateOrProvince', instance.st),
            ('country', instance.c),
            ('email', u'{0}@{1}'.format(instance.uid, OPENDIRADMIN_DOMAIN).lower()),
            ('altName', u'DNS:{0}.{1}.{2}'.format(instance.uid.lower(), OPENDIRADMIN_USER_PREFIX, OPENDIRADMIN_DOMAIN.lower())),
            ('altName', u'email:{0}@{1}'.format(instance.uid.lower(), OPENDIRADMIN_DOMAIN.lower())),
            ]
    for line in instance.mail.split('\n'):
        if line.strip():
            values.append(('altName', u'email:{0}'.format(line.strip())))
    query = u' '.join([u'--{0} "{1}"'.format(k, v.replace('"', '\\"')) for k, v in values])
    if created:
        command = 'add', 'modify'
    elif instance.regen_key:
        command = 'modify --force', 'add'
    elif instance.regen_certificate:
        command = 'renew', 'add'
    else:
        command = None
    if command:
        try:
            perssl_execute(command[0], instance.uid, query)
            perssl_execute(command[1], instance.uid, query)
        except:
            pass
    return True

post_save.connect(create_perssl_user, Users)

def delete_perssl_user(sender, instance, *args, **kwargs):
    query = u"""delete_principal \\"{principal}\\" """.format(principal=instance.uid)
    return perssl_execute('remove', instance.uid, '')

post_delete.connect(delete_perssl_user, Users)

def create_perssl_service(sender, instance, created, *args, **kwargs):
    name = clean_name(instance.cn)
    basename = name.split('.')[0]
    values = [
            ('role', 'service'),
            ('commonName', u'{0}.{1}.{2}'.format(basename, OPENDIRADMIN_SERVICE_PREFIX, OPENDIRADMIN_DOMAIN).lower()),
            ('organization', OPENDIRADMIN_ORGANIZATION),
            ('organizationalUnit', PERSSL_SERVICE_OU),
            ('locality', PERSSL_SERVICE_L),
            ('stateOrProvince', PERSSL_SERVICE_ST),
            ('country', PERSSL_SERVICE_C),
            ('email', u'{0}@{1}'.format(basename, OPENDIRADMIN_DOMAIN).lower()),
            ('altName', u'email:{0}@{1}'.format(basename, OPENDIRADMIN_DOMAIN).lower()),
            ]
    if instance.applednsname:
        for line in instance.applednsname.split('\n'):
            if not line.strip(): continue
            values.append(('altName', u'DNS:{0}'.format(line.strip())))
    if instance.iphostnumber:
        for ip in instance.iphostnumber.split('\n'):
            if not ip.strip(): continue
            values.append(('altName', u'DNS:{0}'.format(ip.lower().strip())))
    query = u' '.join([u'--{0} "{1}"'.format(k, v.replace('"', '\\"')) for k, v in values])
    if created:
        command = 'add', 'modify'
    elif instance.regen_key:
        command = 'modify --force', 'add'
    elif instance.regen_certificate:
        command = 'renew', 'add'
    else:
        command = None
    if command:
        try:
            perssl_execute(command[0], name, query)
            perssl_execute(command[1], name, query)
        except:
            pass
    return True

post_save.connect(create_perssl_service, Service)

def delete_perssl_service(sender, instance, *args, **kwargs):
    return perssl_execute('remove', clean_name(instance.cn), '')

post_delete.connect(delete_perssl_service, Service)

def create_perssl_computer(sender, instance, created, *args, **kwargs):
    name = clean_name(instance.name)
    cert_name = name
    print cert_name
    if cert_name[-1] == '$':
        cert_name = name[0:-1]
    if cert_name.find('.') == -1:
        cert_name = u'{0}.{1}.{2}'.format(cert_name, OPENDIRADMIN_COMPUTER_PREFIX, OPENDIRADMIN_DOMAIN).lower() 

    basename = name.split('.')[0]
    values = [
            ('role', 'computer'),
            ('commonName', cert_name),
            ('organization', OPENDIRADMIN_ORGANIZATION),
            ('organizationalUnit', PERSSL_COMPUTER_OU),
            ('locality', PERSSL_COMPUTER_L),
            ('stateOrProvince', PERSSL_COMPUTER_ST),
            ('country', PERSSL_COMPUTER_C),
            ('email', u'{0}@{1}'.format(basename, OPENDIRADMIN_DOMAIN).lower()),
            ]
    if instance.iphostnumber:
        for ip in instance.iphostnumber.split('\n'):
            if ip.strip():
                values.append(('altName', u'DNS:{0}'.format(ip.lower().strip())))
    query = u' '.join([u'--{0} "{1}"'.format(k, v.replace('"', '\\"')) for k, v in values])
    if created:
        command = 'add', 'modify'
    elif instance.regen_key:
        command = 'modify --force', 'add'
    elif instance.regen_certificate:
        command = 'renew', 'add'
    else:
        command = None
    if command:
        try:
            perssl_execute(command[0], name, query)
            perssl_execute(command[1], name, query)
        except:
            pass
    return True

post_save.connect(create_perssl_computer, Computer)

def delete_perssl_computer(sender, instance, *args, **kwargs):
    return perssl_execute('remove', clean_name(instance.name), '')

post_delete.connect(delete_perssl_computer, Computer)



