#coding=utf8
import itertools
import re

from django.db import models
from django.db.models import fields, SubfieldBase
from django.db.models.signals import post_save, post_delete
import ldapdb.models
from ldapdb.models.fields import CharField, IntegerField, ListField, ImageField


from coreadmin.fields import MultilineField, DateTimeField, FloatField, BooleanField, OctetStringField, ForeignKey
from OpenDirAdmin.settings import OPENDIRADMIN_BASE, OPENDIRADMIN_MACOSX_HOME_ROOT, OPENDIRADMIN_FIRST_UID_NUMBER, OPENDIRADMIN_DOMAIN
from OpenDirAdmin.settings import KADMIN_REALM, OPENDIRADMIN_FIRST_GID_NUMBER, OPENDIRADMIN_COMPUTER_PREFIX, DHCPD_DEFAULT_CONFIG
from OpenDirectory.models import Computer

CACHE_CONFIG = {}
CACHE_SERVER = {}

regexp_ipv6 = re.compile('^((([0-9A-Fa-f]{1,4}:){7}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f\
]{1,4}:){6}:[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){5}:([0-9A-Fa-f]{1,4}:)?[0-9A-F\
a-f]{1,4})|(([0-9A-Fa-f]{1,4}:){4}:([0-9A-Fa-f]{1,4}:){0,2}[0-9A-Fa-f]{1,4})|(([0\
-9A-Fa-f]{1,4}:){3}:([0-9A-Fa-f]{1,4}:){0,3}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:\
){2}:([0-9A-Fa-f]{1,4}:){0,4}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){6}((b((25[0-5\
])|(1d{2})|(2[0-4]d)|(d{1,2}))b).){3}(b((25[0-5])|(1d{2})|(2[0-4]d)|(d{1,2}))b))|\
(([0-9A-Fa-f]{1,4}:){0,5}:((b((25[0-5])|(1d{2})|(2[0-4]d)|(d{1,2}))b).){3}(b((25[\
0-5])|(1d{2})|(2[0-4]d)|(d{1,2}))b))|(::([0-9A-Fa-f]{1,4}:){0,5}((b((25[0-5])|(1d\
{2})|(2[0-4]d)|(d{1,2}))b).){3}(b((25[0-5])|(1d{2})|(2[0-4]d)|(d{1,2}))b))|([0-9A\
-Fa-f]{1,4}::([0-9A-Fa-f]{1,4}:){0,5}[0-9A-Fa-f]{1,4})|(::([0-9A-Fa-f]{1,4}:){0,6\
}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){1,7}:))$')
regexp_ipv4 = re.compile('^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?).(25[0-5]|2[0-4]\
[0-9]|[01]?[0-9][0-9]?).(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?).(25[0-5]|2[0-4][0-\
9]|[01]?[0-9][0-9]?)$')


ddns_update_styles = (('ad-hoc', 'Ad-Hoc'), ('interim', 'Interim'), ('none', 'None'), )

class DhcpBaseClass(ldapdb.models.Model):
    name = CharField(primary_key=True, db_column='cn', max_length=200, blank=False, help_text='RFC4519: user identifier', unique=True, verbose_name=u'Nom abrégé')
    dhcpStatements = MultilineField(db_column='dhcpStatements', blank=True, unique=False, verbose_name=u'Instructions')
    dhcpOption = MultilineField(db_column='dhcpOption', blank=True, unique=False, verbose_name=u'Options')
    #dhcpBootpFlag = BooleanField(u'Répondre aux requêtes BOOTP', default=True)
    default_lease_time = models.IntegerField('Temps de bail par défaut', default=86400, blank=True, null=True) #dhcpStatements
    max_lease_time = models.IntegerField('Temps de bail maximal', default=604800, blank=True, null=True) #dhcpStatements
    ddns_update_style = models.CharField('DDNS update style', max_length=10, choices=ddns_update_styles, default=ddns_update_styles[0][0], blank=True, null=True)#dhcpStatements
    ddns_domain_name = models.CharField('DDNS domain name', max_length=250, blank=True, null=True)#dhcpStatements
    next_server = models.CharField(u'Prochain serveur', max_length=255, blank=True, null=True) #dhcpStatements
    filename = models.CharField(u'Image de démarrage', max_length=255, blank=True, null=True) #dhcpStatements
    routers = models.CharField('Routeurs', max_length=250, blank=True, null=True, help_text=u"Séparer les adresses des serveurs par des espaces") #options
    domain_name_servers = models.CharField('Serveurs DNS', max_length=250, blank=True, null=True, help_text=u"Séparer les adresses des serveurs par des espaces") #options
    host_name = models.CharField(u'Nom d\'hôte', max_length=250, blank=True, null=True) #options
    broadcast_address = models.GenericIPAddressField(verbose_name=u'Adresse IP de broadcast', null=True, blank=True) #options
    domain_search = models.CharField(u'Domaines de recherche', max_length=255, blank=True, null=True, help_text=u"Séparer les domaines par des espaces") #options
    netbios_name_servers = models.CharField(u'Serveurs de noms NETBIOS', max_length=255, blank=True, null=True, help_text=u"Séparer les serveurs par des espaces") #options
    nntp_server = models.CharField(u'Serveurs de NNTP', max_length=255, blank=True, null=True, help_text=u"Séparer les serveurs par des espaces") #options
    pop_server = models.CharField(u'Serveurs de POP3', max_length=255, blank=True, null=True, help_text=u"Séparer les serveurs par des espaces") #options
    smtp_server = models.CharField(u'Serveurs de SMTP', max_length=255, blank=True, null=True, help_text=u"Séparer les serveurs par des espaces") #options
    subnet_mask = models.GenericIPAddressField(verbose_name=u'Masque de sous-réseau', blank=True, null=True) #options
    tftp_server_name = models.CharField(u'Serveurs de tftp', max_length=255, blank=True, null=True, help_text=u"Un seul nom DNS") #options
    kerberos_servers = models.CharField(u'Serveurs Kerberos', max_length=255, blank=True, null=True, help_text=u"Séparer les serveurs par des virgules") #options
    dhcp_options = ['routers', 'domain-name-servers', 'host-name', 'broadcast-address', 'domain-search', 
            'netbios-name-servers', 'nntp-server', 'pop-server', 'smtp-server', 'subnet-mask',
            'tftp-server-name', 'kerberos-servers', ]
    dhcp_statements = ['default-lease-time', 'max-lease-time', 'ddns-update-style', 'ddns-domain-name', 'next-server', 'filename', ]
           

    class Meta:
        verbose_name = u"""Classe DHCP"""
        verbose_name_plural = u"""Classe DHCP"""
        abstract = True

    def save(self, force_insert=True, statements='', options='', *args, **kwargs):
        for key in self.dhcp_options:
            mykey = key.replace('-', '_')
            if getattr(self, mykey, None):
                options += "{0} {1}\n".format(key, getattr(self, mykey))
        for key in self.dhcp_statements:
            mykey = key.replace('-', '_')
            if getattr(self, mykey, None):
                statements += "{0} {1}\n".format(key, getattr(self, mykey))
        self.dhcpStatements = statements.strip()
        self.dhcpOption = options.strip()
        ldapdb.models.Model.save(self, *args, **kwargs)

    def __init__(self, *args, **kwargs):
        ldapdb.models.Model.__init__(self, *args, **kwargs)
        for line in self.dhcpStatements.split('\n'):
            line = line.strip().split(' ', 1) 
            if len(line) != 2: continue
            key, value = line[0].strip(), line[1].strip()
            if key not in self.dhcp_statements: continue
            self.__setattr__(key.replace('-', '_'), value)
        for line in self.dhcpOption.split('\n'):
            line = line.strip().split(' ', 1) 
            if len(line) != 2: continue
            key, value = line[0].strip(), line[1].strip()
            if key not in self.dhcp_options: continue
            self.__setattr__(key.replace('-', '_'), value)

    def __unicode__(self):
        return self.name


class DhcpHost(DhcpBaseClass):
    base_dn = 'cn=DHCP,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'dhcpHost']
    config = models.ForeignKey('DhcpConfig', to_field='name', max_length=255, verbose_name='Configuration')
    dhcpHWAddress = CharField(db_column='dhcpHWAddress', max_length=200, blank=False, unique=False, verbose_name=u'Adresse MAC')
    ip4_address = models.GenericIPAddressField(verbose_name='Adresse IP v4', protocol='IPv4', blank=True)
    ip6_address = models.GenericIPAddressField(verbose_name='Adresse IP v6', protocol='IPv6', blank=True)
    ip6_address_ula = models.GenericIPAddressField(verbose_name='Adresse IP v6 ULA', protocol='IPv6', blank=True)

    def build_dn(self):
        """
        Build the Distinguished Name for this entry.
        """
        return "%s,cn=%s,%s" % (self.build_rdn(), self.config.name, self.base_dn)
        raise Exception("Could not build Distinguished Name")

    class Meta:
        verbose_name = u"""Client DHCP"""
        verbose_name_plural = u"""Clients DHCP"""

    def save(self, *args, **kwargs):
        statements = ''
        if self.ip4_address:
            statements += 'fixed-address ' + unicode(self.ip4_address) + "\n"
        if self.ip6_address:
            statements += 'fixed-address ' + unicode(self.ip6_address) + "\n"
        if self.ip6_address_ula:
            statements += 'fixed-address ' + unicode(self.ip6_address) + "\n"
        DhcpBaseClass.save(self, statements=statements, *args, **kwargs)

    def __init__(self, *args, **kwargs):
        DhcpBaseClass.__init__(self, *args, **kwargs)
        comp = self.dn.split(',', 2)
        if len(comp) == 3:
            comp = self.dn.split(',', 2)[1].split('=', 1)
            if len(comp) == 2:
                l = DhcpConfig.objects.filter(name=comp[1])[0:1]
                if len(l) == 1:
                    self.config = l[0]
        for line in self.dhcpStatements.split('\n'):
            if line.find('fixed-address') < 0:
                continue
            line = line[line.find('fixed-address') + 14:].strip()
            if regexp_ipv4.match(line):
                self.ip4_address = line
            elif regexp_ipv6.match(line):
                if line[0:4] == '2001':
                    self.ip6_address = line
                else:
                    self.ip6_address_ula = line

    def __unicode__(self):
        return self.name


class DhcpSubnet(DhcpBaseClass):
    base_dn = 'cn=DHCP,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'dhcpSubnet', 'dhcpOptions']
    config = models.ForeignKey('DhcpConfig', to_field='name', max_length=255, verbose_name='Configuration')
    dhcpNetMask = IntegerField(db_column='dhcpNetMask', blank=False, verbose_name='Longueur du masque')
    dhcpRange = CharField(db_column='dhcpRange', blank=True, verbose_name='Adresses IP')
    start_address = models.GenericIPAddressField(verbose_name=u'Adresse IP de début', blank=False)
    end_address = models.GenericIPAddressField(verbose_name=u'Adresse IP de fin', blank=False)

    def build_dn(self):
        """
        Build the Distinguished Name for this entry.
        """
        return "%s,cn=%s,%s" % (self.build_rdn(), self.config.name, self.base_dn)
        raise Exception("Could not build Distinguished Name")

    class Meta:
        verbose_name = u"""Sous-réseau DHCP"""
        verbose_name_plural = u"""Sous-réseaux DHCP"""

    def save(self, *args, **kwargs):
        self.dhcpRange = '%s %s' % (unicode(self.start_address), unicode(self.end_address))
        DhcpBaseClass.save(self, *args, **kwargs)

    def __init__(self, *args, **kwargs):
        DhcpBaseClass.__init__(self, *args, **kwargs)
        comp = self.dn.split(',', 2)
        if len(comp) == 3:
            comp = self.dn.split(',', 2)[1].split('=', 1)
            if len(comp) == 2:
                l = DhcpConfig.objects.filter(name=comp[1])[0:1]
                if len(l) == 1:
                    self.config = l[0]
        if self.dhcpRange and self.dhcpRange.find(' ') >= 0:
            a, b = self.dhcpRange.split(' ', 1)
            self.start_address, self.end_address = a.strip(), b.strip()

    def __unicode__(self):
        return unicode(self.name)


class DhcpServer(ldapdb.models.Model):
    base_dn = 'cn=DHCP,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'dhcpServer']
    name = CharField(primary_key=True, db_column='cn', max_length=200, blank=False, help_text='RFC4519: user identifier', unique=True, verbose_name=u'Nom abrégé', default='server')
    config = models.ForeignKey('DhcpConfig', to_field='name', max_length=255, verbose_name='Configuration')
    dhcpServiceDN = CharField('Emplacement de la configuration', db_column='dhcpServiceDN', max_length=200, blank=True) #dhcpServiceDN: cn=DHCP_config,cn=DHCP,OPENDIRADMIN_BASE
    class Meta:
        verbose_name = u"""Serveur DHCP"""
        verbose_name_plural = u"""Serveurs DHCP"""

    def save(self, *args, **kwargs):
        if self.config:
            self.dhcpServiceDN = self.config.dn
        ldapdb.models.Model.save(self, *args, **kwargs)

    def __init__(self, *args, **kwargs):
        ldapdb.models.Model.__init__(self, *args, **kwargs)
        CACHE_SERVER[self.name] = self
        if self.dhcpServiceDN:
            f = self.dhcpServiceDN.split(',', 1)[0].split('=', 1)
            if len(f) == 2:
                if f[1] not in CACHE_CONFIG:
                    l = DhcpConfig.objects.filter(name=f[1])[0:1]
                    if len(l) == 1:
                        self.config = l[0]
                else:
                    self.config = CACHE_CONFIG[f[1]]

    def __unicode__(self):
        return self.name


class DhcpConfig(DhcpBaseClass):
    base_dn = 'cn=DHCP,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'dhcpService', 'dhcpOptions']
    dhcpPrimaryDN = CharField('Emplacement du serveur primaire', db_column='dhcpPrimaryDN', max_length=200, null=True, blank=True) #dhcpServiceDN: cn=server,cn=DHCP,OPENDIRADMIN_BASE
    dhcpSecondaryDN = CharField('Emplacement du serveur secondaire', db_column='dhcpSecondaryDN', max_length=200, null=True, blank=True) #dhcpServiceDN: cn=server,cn=DHCP,OPENDIRADMIN_BASE
    domain_name = models.CharField('Nom de domaine', max_length=250)
    primary = models.ForeignKey(DhcpServer, to_field='name', max_length=255, verbose_name='Serveur primaire', null=True, blank=True, related_name='primary')
    secondary = models.ForeignKey(DhcpServer, to_field='name', max_length=255, verbose_name='Serveur secondaire', null=True, blank=True, related_name='secondary')

    def save(self, *args, **kwargs):
        options = []
        if self.primary:
            self.dhcpPrimaryDN = self.primary.dn
        if self.secondary:
            self.dhcpSecondaryDN = self.secondary.dn
        if self.domain_name:
            options.append('domain-name ' + unicode(self.domain_name))
        DhcpBaseClass.save(self, options="\n".join(options) + "\n", *args, **kwargs)

    def __init__(self, *args, **kwargs):
        DhcpBaseClass.__init__(self, *args, **kwargs)
        CACHE_CONFIG[self.name] = self
        for line in self.dhcpOption.split('\n'):
            line = line.strip().split(' ', 1) 
            if len(line) != 2: continue
            key, value = line[0].strip(), line[1].strip()
            if key != 'domain-name': continue
            self.__setattr__(key.replace('-', '_'), value)
        if self.dhcpPrimaryDN:
            f = self.dhcpPrimaryDN.split(',', 1)[0].split('=', 1)
            if len(f) == 2:
                if f[1] not in CACHE_SERVER:
                    l = DhcpServer.objects.filter(name=f[1])[0:1]
                    if len(l) == 1:
                        self.primary = l[0]
                else:
                    self.primary = CACHE_SERVER[f[1]]
        if self.dhcpSecondaryDN:
            f = self.dhcpSecondaryDN.split(',', 1)[0].split('=', 1)
            if len(f) == 2:
                if f[1] not in CACHE_SERVER:
                    l = DhcpServer.objects.filter(name=f[1])[0:1]
                    if len(l) == 1:
                        self.secondary = l[0]
                else:
                    self.secondary = CACHE_SERVER[f[1]]

    class Meta:
        verbose_name = u"""Configuration DHCP"""
        verbose_name_plural = u"""Configurations DHCP"""

    def __unicode__(self):
        return unicode(self.name)


def clean_computer_name(name):
    if name[-1] == '$':
        name = name[0:-1]
    if name.find('.') == -1:
        name = u'{0}.{1}.{2}'.format(basename, OPENDIRADMIN_COMPUTER_PREFIX, OPENDIRADMIN_DOMAIN).lower() 
    return name
 
def create_dhcpd_computer(sender, instance, created, *args, **kwargs):
    config, c = DhcpConfig.objects.get_or_create(name=DHCPD_DEFAULT_CONFIG)
    name = clean_computer_name(instance.name)
    o__name = clean_computer_name(instance.o__name)
    if o__name != name:
        DhcpHost.objects.filter(name__startswith=o__name).delete()
    if not instance.macaddress or not instance.iphostnumber:
        return True
    o__mac_list = set(x.strip().lower() for x in instance.o__macaddress.split('\n'))
    mac_list = set(x.strip().lower() for x in instance.macaddress.split('\n'))
    for mac in o__mac_list - mac_list: # remove old MAC addresses
        DhcpHost.objects.filter(name=u'{0}_{1}'.format(o__name, mac)).delete()

    for mac, ip in itertools.izip(instance.macaddress.split('\n'), instance.iphostnumber.split('\n')):
        mac, ip = mac.strip(), ip.strip()
        full_name = u'{0}_{1}'.format(name, mac)
        hosts = DhcpHost.objects.filter(name=full_name, dhcpHWAddress=mac)[0:1]
        if len(hosts) == 1:
            host = hosts[0]
        else:
            host = DhcpHost(name=full_name, dhcpHWAddress=mac, config=config)
        if regexp_ipv4.match(ip):
            host.ip4_address = ip
        elif ip[0:4] == '2001':
            host.ip6_address = ip
        else:
            host.ip6_address_ula = ip
        host.host_name = name
        host.save()
    return True

post_save.connect(create_dhcpd_computer, Computer)

def delete_dhcpd_computer(sender, instance, *args, **kwargs):
    name = clean_computer_name(instance.name)
    DhcpHost.objects.filter(name__startswith=name).delete()

post_delete.connect(delete_dhcpd_computer, Computer)


