#coding=utf8
import subprocess

from django.db import models
from django.db.models.signals import post_save, post_delete
from OpenDirectory.models import Users, Service
from OpenDirAdmin.settings import KADMIN_USER, KADMIN_PASSWORD, KADMIN_HOST, KADMIN_PORT, KADMIN_REALM, KADMIN_KEYTAB, KADMIN_VERSION



def execute_kerberos(query):
    if KADMIN_VERSION == 'mit':
        kerberos_line = u"""kadmin -p "{user}" -r "{realm}" -w "{password}" -s {host} {port} -q "{query}" """.format(
                user=KADMIN_USER, realm=KADMIN_REALM, password=KADMIN_PASSWORD, host=KADMIN_HOST, port=KADMIN_PORT, query=query)
    else:
        kerberos_line = u"""kadmin -p "{user}" -r "{realm}" -K "{keytab}" -a "{host}" -s {port}  "{query}" """.format(
                user=KADMIN_USER, realm=KADMIN_REALM, keytab=KADMIN_KEYTAB, host=KADMIN_HOST, port=KADMIN_PORT, query=query)
    try:
        subprocess.check_call(kerberos_line, shell=True)
        result = True
    except:
        result = False
    return result

def create_kerberos_user(sender, instance, created, *args, **kwargs):
    if KADMIN_VERSION == 'mit':
        query = u"""add_principal -pw \\"{pwd}\\" \\"{principal}\\" """.format(principal=instance.uid, pwd=instance.userpassword) 
    else:
        query = u"""add --password \\"{pwd}\\" \\"{principal}\\" """.format(principal=instance.uid, pwd=instance.userpassword) 
    execute_kerberos(query)
    if KADMIN_VERSION == 'mit':
        query = u"""change_password -pw \\"{pwd}\\" \\"{principal}\\" """.format(principal=instance.uid, pwd=instance.userpassword)
    else:
        query = u"""passwd --password \\"{pwd}\\" \\"{principal}\\" """.format(principal=instance.uid, pwd=instance.userpassword)
    return execute_kerberos(query)

post_save.connect(create_kerberos_user, Users)

def delete_kerberos_user(sender, instance, *args, **kwargs):
    query = u"""delete_principal \\"{principal}\\" """.format(principal=instance.uid)
    return execute_kerberos(query)

post_delete.connect(delete_kerberos_user, Users)

def create_kerberos_service(sender, instance, created, *args, **kwargs):
    if created:
        if KADMIN_VERSION == 'mit':
            query = u"""add_principal -randkey \\"{principal}\\" """.format(principal=instance.cn) 
        else:
            query = u"""add --random-key \\"{principal}\\" """.format(principal=instance.cn) 
        return execute_kerberos(query)
    elif instance.regen_keytab:
        if KADMIN_VERSION == 'mit':
            query = u"""change_password -randkey \\"{principal}\\" """.format(principal=instance.cn)
        else:
            query = u"""passwd --random-key \\"{principal}\\" """.format(principal=instance.cn)
        return execute_kerberos(query)

post_save.connect(create_kerberos_service, Service)

def delete_kerberos_service(sender, instance, *args, **kwargs):
    query = u"""delete_principal \\"{principal}\\" """.format(principal=instance.cn)
    return execute_kerberos(query)

post_delete.connect(delete_kerberos_service, Service)



