#coding=utf8

from django.contrib import admin
from django.core.urlresolvers import reverse
from django import forms

import coreadmin.fields
from OpenDirectory.models import *
def ca_cert(self): return '<a href="%s">CA</span>' % (reverse('Certificates.views.get_cacert'))
ca_cert.allow_tags = True
ca_cert.short_description = u'CA'

class CoreModelAdmin(admin.ModelAdmin):
    exclude = ['dn', ]
    formfield_overrides = {
            coreadmin.fields.MultilineField: {'widget': forms.Textarea},
            coreadmin.fields.OctetStringField: {'widget': forms.Textarea},
            }


class UsersAdmin(CoreModelAdmin):
    exclude_fields = ['applegenerateduid', 'altsecurityidentities', 'appleuserprintattribute', 'appleusermailattribute', 'applemcxflags', 'authauthority', 'cn', ]
    fieldsets = [
        (None, {'fields': [ ('givenname', 'sn', 'applenamesuffix', ),
            ('uid', 'cn', ), 'userpassword', ('uidnumber', 'gidnumber',), 'description', ('regen_certificate', 'regen_key', )]}),
        (u'Infos', {'fields': [ 'street', ('l', 'st', 'postalcode', 'c', ),
            ('applecompany', 'title', 'departmentnumber', ),
            ('labeleduri', 'homephone', 'telephonenumber', 'applewebloguri', ),
            ('mail', 'appleimhandle', ), 'appleuserpicture', 'applebirthday', 'applekeyword', 
            ], 'classes': ('collapse', )}),
        (u'Mac OS X', {'fields': ['homedirectory', ], 'classes': ('collapse', )}),
        (u'UNIX', {'fields': ['loginshell', ], 'classes': ('collapse', )}),
        (u'Windows', {'fields': ['scriptpath', 'homedrive', 'smbhome', 'profilepath', ], 'classes': ('collapse', )}),
        ]
    def pkcs12(self): return '<a href="%s">PKCS12</span>' % (reverse('Certificates.views.get_pkcs12', kwargs={'name': self.uid}))
    pkcs12.allow_tags = True
    pkcs12.short_description = u'PKCS12'
    def crt(self): return '<a href="%s">Certificate</span>' % (reverse('Certificates.views.get_info', kwargs={'name': self.uid, 'cat': 'users', 'info': 'crt'}))
    crt.allow_tags = True
    crt.short_description = u'Certificate'
    def key(self): return '<a href="%s">Key</span>' % (reverse('Certificates.views.get_info', kwargs={'name': self.uid, 'cat': 'users', 'info': 'key'}))
    key.allow_tags = True
    key.short_description = u'Key'
    list_display = ['uid', pkcs12, crt, key, ca_cert]


class GroupsAdmin(CoreModelAdmin):
    exclude_fields = ['applegenerateduid', 'appleownerguid', 'applegroupmemberguid']
    fieldsets = [
            (None, {'fields': ['applegrouprealname', 'cn', 'appleuserpicture', 'gidnumber', 'applegrouphomeowner', 'description', 'memberuid', ]})
            ]


class ComputerGroupsAdmin(CoreModelAdmin):
    exclude_fields = ['applegenerateduid', 'applegroupmemberguid']
    fieldsets = [
            (None, {'fields': [('applegrouprealname', 'cn',), 'gidnumber', 'description', 'memberuid', ]})
            ]


class ServiceAdmin(CoreModelAdmin):
    def keytab(self):
            return '<a href="%s">keytab</span>' % (reverse('Kerberos.views.get_keytab', kwargs={'cn': self.cn}))
    keytab.allow_tags = True
    keytab.short_description = u'Keytab'
    def crt(self): return '<a href="%s">Certificate</span>' % (reverse('Certificates.views.get_info', kwargs={'name': self.cn, 'cat': 'service', 'info': 'crt'}))
    crt.allow_tags = True
    crt.short_description = u'Certificate'
    def key(self): return '<a href="%s">Key</span>' % (reverse('Certificates.views.get_info', kwargs={'name': self.cn, 'cat': 'service', 'info': 'key'}))
    key.allow_tags = True
    key.short_description = u'Key'
    fields = [ 'cn', ('applednsname', 'iphostnumber', ), ('appleservicetype', 'appleserviceurl', 'appleserviceport'), 'description', ('regen_certificate', 'regen_key', 'regen_keytab'), ]
    exclude_fields = [ 'applekeyword', 'appleserviceslocator', 'appleserviceinfo', 'userpkcs12', 'usercertificate' ]
    list_display = ['cn', 'applednsname', 'iphostnumber', keytab, crt, key, ca_cert]



class ComputerAdmin(CoreModelAdmin):
    fieldsets = [
        (None, {'fields': [ ('name', 'macaddress', 'iphostnumber', 'applehwuuid', ),
            ('appleownerguid', 'gidnumber'), ('regen_certificate', 'regen_key', ),  'description', 'applerealname', 
            'applekeyword', 'appleserviceurl', ]})
        ]
    def crt(self): return '<a href="%s">Certificate</span>' % (reverse('Certificates.views.get_info', kwargs={'name': self.name, 'cat': 'computer', 'info': 'crt'}))
    crt.allow_tags = True
    crt.short_description = u'Certificate'
    def key(self): return '<a href="%s">Key</span>' % (reverse('Certificates.views.get_info', kwargs={'name': self.name, 'cat': 'computer', 'info': 'key'}))
    key.allow_tags = True
    key.short_description = u'Key'
    list_display = ['name', crt, key, ca_cert]



#admin.site.register(CertificateAuthorities, CoreModelAdmin)
admin.site.register(ComputerGroups, ComputerGroupsAdmin)
admin.site.register(Computer, ComputerAdmin)
admin.site.register(Config, CoreModelAdmin)
admin.site.register(Groups, GroupsAdmin)
admin.site.register(Users, UsersAdmin)
admin.site.register(Mounts, CoreModelAdmin)
admin.site.register(Service, ServiceAdmin)
#admin.site.register(PresetComputerGroups, CoreModelAdmin)
#admin.site.register(PresetComputers, CoreModelAdmin)
#admin.site.register(PresetGroups, CoreModelAdmin)
#admin.site.register(PresetUsers, CoreModelAdmin)
