#coding=utf8
import os.path
import uuid

from django.db import models
from django.db.models import fields, SubfieldBase
import ldapdb.models
from ldapdb.models.fields import CharField, IntegerField, ListField, ImageField

from coreadmin.fields import MultilineField, DateTimeField, FloatField, BooleanField, OctetStringField, ForeignKey
from OpenDirAdmin.settings import OPENDIRADMIN_BASE, OPENDIRADMIN_MACOSX_HOME_ROOT, OPENDIRADMIN_FIRST_UID_NUMBER, OPENDIRADMIN_DOMAIN, KADMIN_REALM, OPENDIRADMIN_FIRST_GID_NUMBER

def groups_iterable():
    for group in Groups.objects.order_by('cn'):
        yield (group.gidnumber, group.cn)

BOOLEAN_CHOICES = ((False, u'Non'), (True, u'Oui'), )


class Users(ldapdb.models.Model):
    u"""
            
        """
    base_dn = 'cn=users,' + OPENDIRADMIN_BASE

    object_classes = ['top', 'person', 'inetOrgPerson', 'organizationalPerson', 'posixAccount', 'shadowAccount', 'apple-user', 'extensibleObject']
    uid = CharField(primary_key=True, db_column='uid', max_length=200, blank=False, help_text='RFC4519: user identifier', unique=True, verbose_name=u'Nom abrégé')
    c = CharField(db_column='c', max_length=200, blank=True, help_text='RFC2256: ISO-3166 country 2-letter code', unique=False, verbose_name=u'Pays')
    #gidnumber = IntegerField(db_column='gidNumber', blank=True, help_text=u'Laisser vide pour créer un groupe automatiquement', choices=groups_iterable(), verbose_name=u'Numéro de groupe principal')
    gidnumber = ForeignKey('Groups', to_field='gidnumber', db_column='gidNumber', null=True, blank=True, help_text=u'Laisser vide pour créer un groupe automatiquement', verbose_name=u'Groupe primaire')
    applegenerateduid = CharField(db_column='apple-generateduid', max_length=200, blank=True, help_text='generated unique ID', unique=True, verbose_name=u'apple-generateduid')
    appleimhandle = MultilineField(db_column='apple-imhandle', max_length=200, blank=True, help_text='IM handle (service:account name)', unique=False, verbose_name=u'Messagerie instantanée')
    applewebloguri = CharField(db_column='apple-webloguri', max_length=200, blank=True, help_text='Weblog URI', unique=False, verbose_name=u'Journal web')
    profilepath = CharField(db_column='profilePath', max_length=200, blank=True, help_text='NT profilePath', unique=False, verbose_name=u'profilePath')
    st = CharField(db_column='st', max_length=200, blank=True, help_text='RFC2256: state or province which this object resides in', unique=False, verbose_name=u'Région / province')
    cn = CharField(db_column='cn', max_length=200, blank=False, help_text='RFC4519: common name(s) for which the entity is known by', unique=False, verbose_name=u'Nom commun')
    appleusermailattribute = CharField(db_column='apple-user-mailattribute', max_length=200, blank=True, help_text='mail attribute', unique=False, verbose_name=u'apple-user-mailattribute')
    homedrive = CharField(db_column='homeDrive', max_length=200, blank=True, help_text='NT homeDrive', unique=False, verbose_name=u'homeDrive')
    authauthority = MultilineField(db_column='authAuthority', max_length=200, blank=True, help_text='password server authentication authority', unique=False, verbose_name=u'authAuthority')
    applecompany = CharField(db_column='apple-company', max_length=200, blank=True, help_text='company', unique=False, verbose_name=u'Entreprise')
    userpassword = CharField(db_column='userPassword', max_length=200, blank=True, help_text='RFC4519/2307: password of user', unique=False, verbose_name=u'Mot de passe')
    l = CharField(db_column='l', max_length=200, blank=True, help_text='RFC2256: locality which this object resides in', unique=False, verbose_name=u'Ville')
    uidnumber = IntegerField(db_column='uidNumber', blank=True, help_text='RFC2307: An integer uniquely identifying a user in an administrative domain', unique=True, verbose_name=u'Identifiant unique')
    description = CharField(db_column='description', max_length=200, blank=True, help_text='RFC4519: descriptive information', unique=False, verbose_name=u'Commentaire')
    applemcxflags = CharField(db_column='apple-mcxflags', max_length=200, blank=True, help_text='mcx flags', unique=False, verbose_name=u'apple-mcxflags')
    givenname = CharField(db_column='givenName', max_length=200, blank=True, help_text='RFC2256: first name(s) for which the entity is known by', unique=False, verbose_name=u'Prénom')
    applenamesuffix = CharField(db_column='apple-namesuffix', max_length=200, blank=True, help_text='namesuffix', unique=False, verbose_name=u'Suffixe')
    homedirectory = CharField(db_column='homeDirectory', max_length=200, blank=True, help_text='The absolute path to the home directory', unique=False, verbose_name=u'Dossier de départ')
    labeleduri = CharField(db_column='labeledURI', max_length=200, blank=True, help_text='RFC2079: Uniform Resource Identifier with optional label', unique=False, verbose_name=u'Page web')
    street = CharField(db_column='street', max_length=200, blank=True, help_text='RFC2256: street address of this object', unique=False, verbose_name=u'Rue')
    scriptpath = CharField(db_column='scriptPath', max_length=200, blank=True, help_text='NT scriptPath', unique=False, verbose_name=u'scriptPath')
    sn = CharField(db_column='sn', max_length=200, blank=False, help_text='RFC2256: last (family) name(s) for which the entity is known by', unique=False, verbose_name=u'Nom de famille')
    homephone = CharField(db_column='homePhone', max_length=200, blank=True, help_text='RFC1274: home telephone number', unique=False, verbose_name=u'homePhone')
    loginshell = CharField(db_column='loginShell', max_length=200, blank=True, help_text='The path to the login shell', unique=False, verbose_name=u'loginShell', default='/bin/bash')
    applekeyword = MultilineField(db_column='apple-keyword', max_length=200, blank=True, help_text='keywords', unique=False, verbose_name=u'Mots-clef')
    appleuserprintattribute = CharField(db_column='apple-user-printattribute', max_length=200, blank=True, help_text='print attribute', unique=False, verbose_name=u'apple-user-printattribute')
    postalcode = CharField(db_column='postalCode', max_length=200, blank=True, help_text='RFC2256: postal code', unique=False, verbose_name=u'Code postal')
    mail = MultilineField(db_column='mail', max_length=200, blank=True, help_text='RFC1274: RFC822 Mailbox', unique=False, verbose_name=u'mail')
    departmentnumber = CharField(db_column='departmentNumber', max_length=200, blank=True, help_text='RFC2798: identifies a department within an organization', unique=False, verbose_name=u'Service')
    title = CharField(db_column='title', max_length=200, blank=True, help_text='RFC2256: title associated with the entity', unique=False, verbose_name=u'Profession')
    telephonenumber = CharField(db_column='telephoneNumber', max_length=200, blank=True, help_text='RFC2256: Telephone Number', unique=False, verbose_name=u'Téléphone')
    altsecurityidentities = MultilineField(db_column='altSecurityIdentities', max_length=200, blank=True, help_text='None', unique=False, verbose_name=u'altSecurityIdentities')
    smbhome = CharField(db_column='smbHome', max_length=200, blank=True, help_text='smbHome', unique=False, verbose_name=u'smbHome')
    applecertificatesigningrequest = OctetStringField(db_column='apple-certificateSigningRequest', max_length=200, blank=True, help_text='None', unique=False, verbose_name=u'apple-certificateSigningRequest')
    appleuserpicture = CharField(db_column='apple-user-picture', max_length=200, blank=True, help_text='picture', unique=False, verbose_name=u'Image')
    applebirthday = DateTimeField(db_column='apple-birthday', blank=True, help_text='Birthday', unique=False, verbose_name=u'Anniversaire')
    userpkcs12 = OctetStringField(db_column='userPKCS12', max_length=200, blank=True, help_text='RFC2798: personal identity information, a PKCS #12 PFX', unique=False, verbose_name=u'userPKCS12')
    usercertificate = OctetStringField(db_column='userCertificate', max_length=200, blank=True, help_text='RFC2256: X.509 user certificate, use ;binary', unique=False, verbose_name=u'userCertificate')
    cacertificate = OctetStringField(db_column='cACertificate', max_length=200, blank=True, help_text='RFC2256: X.509 CA certificate, use ;binary', unique=False, verbose_name=u'cACertificate')

    regen_certificate = models.BooleanField(u'Regénérer le certificat', max_length=20, default=BOOLEAN_CHOICES[0][0], choices=BOOLEAN_CHOICES, blank=True)
    regen_key = models.BooleanField(u'Regénérer la clef privée', max_length=20, default=BOOLEAN_CHOICES[0][0], choices=BOOLEAN_CHOICES, blank=True)
#krbPrincipalName, krbPrincipalKey, krbLastPwdChange, and krbExtraData
    class Meta:
        verbose_name = u"""Utilisateur"""
        verbose_name_plural = u"""Utilisateurs"""

    def __init__(self, *args, **kwargs):
        super(Users, self).__init__(*args, **kwargs)
        if 'userpassword' in self.__dict__:
            self.init_password = self.userpassword
        else:
            self.init_password = None

    def save(self, *args, **kwargs):
        self.is_new = not self.applegenerateduid
        if self.is_new:
            self.applegenerateduid = unicode(uuid.uuid1())
        if not self.uidnumber:
            l = Users.objects.order_by('-uidnumber')[0:1]
            if len(l) == 1:
                self.uidnumber = l[0].uidnumber + 1
            else:
                self.uidnumber = OPENDIRADMIN_FIRST_UID_NUMBER + 1
        if not self.cn:
            self.cn = self.givenname + u' ' + self.sn
        if not self.cn:
            self.cn = 'Common Name'
        if not self.gidnumber:
            groups = Groups.objects.filter(cn=self.uid)[0:1]
            if len(groups) == 1:
                group = groups[0]
            else:
                group = Groups(cn=self.uid)
                group.save()
            self.gidnumber = group
        if not self.homedirectory:
            self.homedirectory = os.path.join(OPENDIRADMIN_MACOSX_HOME_ROOT, self.uid)
        if not self.altsecurityidentities:
            self.altsecurityidentities = 'Kerberos:{0}@{1}'.format(self.uid, KADMIN_REALM)
        if not self.authauthority:
            self.authauthority = ';Kerberosv5;;{0}@{1};{1};'.format(self.uid, KADMIN_REALM)
        super(Users, self).save(*args, **kwargs)

    def __str__(self):
        return self.uid

    def __unicode__(self):
        return self.uid


class Groups(ldapdb.models.Model):
    u"""
            
        """    
    base_dn = 'cn=groups,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'posixGroup', 'extensibleObject', 'apple-group']
    cn = CharField(primary_key=True, db_column='cn', max_length=200, blank=False, help_text='RFC4519: common name(s) for which the entity is known by', unique=True, verbose_name=u'Nom abrégé')
    description = CharField(db_column='description', max_length=200, blank=True, help_text='RFC4519: descriptive information', unique=False, verbose_name=u'Description')
    applegenerateduid = CharField(db_column='apple-generateduid', max_length=200, blank=True, help_text='generated unique ID', unique=False, verbose_name=u'apple-generateduid')
    gidnumber = IntegerField(unique=True, db_column='gidNumber', blank=True, help_text='RFC2307: An integer uniquely identifying a group in an administrative domain', verbose_name=u'gidNumber')
    appleownerguid = CharField(db_column='apple-ownerguid', max_length=200, blank=True, help_text='owner GUID', unique=False, verbose_name=u'Propriétaire')
    appleuserpicture = CharField(db_column='apple-user-picture', max_length=200, blank=True, help_text='picture', unique=False, verbose_name=u'Image du groupe')
    applegrouphomeowner = CharField(db_column='apple-group-homeowner', max_length=200, blank=True, help_text='group home owner settings', unique=False, verbose_name=u'Propriétaire du groupe')
    applegrouprealname = CharField(db_column='apple-group-realname', max_length=200, blank=True, help_text='group real name', unique=False, verbose_name=u'Nom du groupe')
    applegroupmemberguid = CharField(db_column='apple-group-memberguid', max_length=200, blank=True, help_text='group real name', unique=False, verbose_name=u'apple-group-memberguid')
    memberuid = MultilineField(db_column='memberUid', max_length=200, blank=True, help_text='None', unique=False, verbose_name=u'memberUid')

    class Meta:
        verbose_name = u"""Groupe"""
        verbose_name_plural = u"""Groupes"""

    def save(self, *args, **kwargs):
        if not self.applegenerateduid:
            self.applegenerateduid = unicode(uuid.uuid1())
        if not self.gidnumber:
            l = Groups.objects.order_by('-gidnumber')[0:1]
            if len(l) == 1:
                self.gidnumber = l[0].gidnumber + 1
            else:
                self.gidnumber = OPENDIRADMIN_FIRST_GID_NUMBER + 1
        super(Groups, self).save(*args, **kwargs)

    def __str__(self):
        return self.cn

    def __unicode__(self):
        return self.cn


class CertificateAuthorities(ldapdb.models.Model):
    u"""
            
        """    
    base_dn = 'cn=certificateauthorities,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'pkiCA', 'organizationalUnit', ]
    name = CharField(primary_key=True, db_column='ou', max_length=200, blank=False, help_text='RFC2256: organizational unit this object belongs to', unique=True, verbose_name=u'Nom')
    location = CharField(db_column='l', max_length=200, blank=True, help_text='RFC2256: locality which this object resides in', unique=False, verbose_name=u'Ville')
    st = CharField(db_column='st', max_length=200, blank=True, help_text='RFC2256: state or province which this object resides in', unique=False, verbose_name=u'Région / province')
    cacertificate = OctetStringField(db_column='cACertificate', max_length=200, blank=True, help_text='RFC2256: X.509 CA certificate, use ;binary', unique=False, verbose_name=u'cACertificate')
    authorityrevocationlist = OctetStringField(db_column='authorityRevocationList', max_length=200, blank=True, help_text='RFC2256: X.509 authority revocation list, use ;binary', unique=False, verbose_name=u'authorityRevocationList')
    certificaterevocationlist = OctetStringField(db_column='certificateRevocationList', max_length=200, blank=True, help_text='RFC2256: X.509 certificate revocation list, use ;binary', unique=False, verbose_name=u'certificateRevocationList')


    class Meta:
        verbose_name = u"""Autorité de certification"""
        verbose_name_plural = u"""Autorités de certification"""

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name


class Config(ldapdb.models.Model):
    u"""
            
        """    
    base_dn = 'cn=config,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'apple-configuration']
    name = CharField(primary_key=True, db_column='cn', max_length=200, blank=True, help_text='RFC4519: common name(s) for which the entity is known by', unique=True, verbose_name=u'Nom')
    applepasswordserverlocation = CharField(db_column='apple-password-server-location', max_length=200, blank=True, help_text='password server location', unique=False, verbose_name=u'apple-password-server-location')
    appledatastamp = CharField(db_column='apple-data-stamp', max_length=200, blank=True, help_text='data stamp', unique=False, verbose_name=u'apple-data-stamp')
    appleconfigrealname = CharField(db_column='apple-config-realname', max_length=200, blank=True, help_text='config real name', unique=False, verbose_name=u'apple-config-realname')
    applepasswordserverlist = CharField(db_column='apple-password-server-list', max_length=200, blank=True, help_text='password server replication plist', unique=False, verbose_name=u'apple-password-server-list')
    appleldapreplica = CharField(db_column='apple-ldap-replica', max_length=200, blank=True, help_text='LDAP replication list', unique=False, verbose_name=u'apple-ldap-replica')
    appleldapwritablereplica = CharField(db_column='apple-ldap-writable-replica', max_length=200, blank=True, help_text='LDAP writable replication list', unique=False, verbose_name=u'apple-ldap-writable-replica')
    applekdcauthkey = CharField(db_column='apple-kdc-authkey', max_length=200, blank=True, help_text='KDC master key RSA encrypted with realm public key', unique=False, verbose_name=u'apple-kdc-authkey')
    applekdcconfigdata = CharField(db_column='apple-kdc-configdata', max_length=200, blank=True, help_text='Contents of the kdc.conf file', unique=False, verbose_name=u'apple-kdc-configdata')
    applelastserverid = IntegerField(db_column='apple-last-serverid', blank=True, help_text='Last serverID used', unique=False, verbose_name=u'apple-last-serverid')
    applexmlplist = CharField(db_column='apple-xmlplist', max_length=200, blank=True, help_text='XML plist data', unique=False, verbose_name=u'apple-xmlplist')
    class Meta:
        verbose_name = u"""Configuration"""
        verbose_name_plural = u"""Configurations"""

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name


class Computer(ldapdb.models.Model):
    u"""
            
        """    
    base_dn = 'cn=computers,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'apple-computer']
    name = CharField(primary_key=True, db_column='cn', max_length=200, blank=False, help_text='RFC4519: common name(s) for which the entity is known by', unique=True, verbose_name=u'Nom')
    macaddress = MultilineField(db_column='macAddress', max_length=200, blank=True, help_text='MAC address', unique=False, verbose_name=u'Adresse MAC')
    #appleownerguid = CharField(db_column='apple-ownerguid', max_length=200, blank=True, help_text='owner GUID', unique=False, verbose_name=u'Propriétaire')
    appleownerguid = ForeignKey(Users, to_field='applegenerateduid', db_column='apple-ownerguid', null=True, blank=True, verbose_name=u'Propriétaire')
    #applenetworkview = ForeignKey(Domains, to_field='name', db_column='apple-networkview', null=True, blank=True, verbose_name=u'Zone DNS')
    description = CharField(db_column='description', max_length=200, blank=True, help_text='RFC4519: descriptive information', unique=False, verbose_name=u'description')
    applerealname = CharField(db_column='apple-realname', max_length=200, blank=True, help_text='real name', unique=False, verbose_name=u'Nom du matériel')
    iphostnumber = MultilineField(db_column='ipHostNumber', max_length=200, blank=True, help_text='IP address', unique=False, verbose_name=u'Adresses IP')
    #gidnumber = IntegerField(db_column='gidNumber', blank=True, help_text='RFC2307: An integer uniquely identifying a group in an administrative domain', unique=False, verbose_name=u'gidNumber')
    gidnumber = ForeignKey('ComputerGroups', to_field='gidnumber', db_column='gidNumber', null=True, blank=True, verbose_name=u'Groupe primaire')
    applehwuuid = CharField(db_column='apple-hwuuid', max_length=200, blank=True, help_text='Hardware uuid of computer', unique=False, verbose_name=u'Numéro de série')
    applekeyword = CharField(db_column='apple-keyword', max_length=200, blank=True, help_text='keywords', unique=False, verbose_name=u'Mots-clefs')
    appleserviceurl = MultilineField(db_column='apple-service-url', max_length=200, blank=True, help_text='URL of service', unique=False, verbose_name=u'URL des services')
    applegenerateduid = CharField(db_column='apple-generateduid', max_length=200, blank=True, help_text='generated unique ID', unique=False, verbose_name=u'apple-generateduid')

    userpkcs12 = OctetStringField(db_column='userPKCS12', max_length=200, blank=True, help_text='RFC2798: personal identity information, a PKCS #12 PFX', unique=False, verbose_name=u'userPKCS12')
    usercertificate = OctetStringField(db_column='userCertificate', max_length=200, blank=True, help_text='RFC2256: X.509 user certificate, use ;binary', unique=False, verbose_name=u'userCertificate')
    regen_certificate = models.BooleanField(u'Regénérer le certificat', max_length=20, default=BOOLEAN_CHOICES[0][0], choices=BOOLEAN_CHOICES, blank=True)
    regen_key = models.BooleanField(u'Regénérer la clef privée', max_length=20, default=BOOLEAN_CHOICES[0][0], choices=BOOLEAN_CHOICES, blank=True)

    def __init__(self, *args, **kwargs):
        ldapdb.models.Model.__init__(self, *args, **kwargs)
        self.o__name = self.name
        self.o__iphostnumber = self.iphostnumber
        self.o__macaddress = self.macaddress

    class Meta:
        verbose_name = u"""Ordinateur"""
        verbose_name_plural = u"""Ordinateurs"""

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.applegenerateduid:
            self.applegenerateduid = unicode(uuid.uuid1())
        ldapdb.models.Model.save(self, *args, **kwargs)


class ComputerGroups(ldapdb.models.Model):
    u"""
            
        """    
    base_dn = 'cn=computer_groups,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'posixGroup', 'extensibleObject', 'apple-group']
    gidnumber = IntegerField(db_column='gidNumber', blank=True, help_text='RFC2307: An integer uniquely identifying a group in an administrative domain', unique=True, verbose_name=u'Numéro de groupe')
    cn = CharField(primary_key=True, db_column='cn', max_length=200, blank=False, help_text='RFC4519: common name(s) for which the entity is known by', unique=True, verbose_name=u'Nom abrégé')
    applegenerateduid = CharField(db_column='apple-generateduid', max_length=200, blank=True, help_text='generated unique ID', unique=False, verbose_name=u'apple-generateduid')
    applegroupmemberguid = CharField(db_column='apple-group-memberguid', max_length=200, blank=True, help_text='group real name', unique=False, verbose_name=u'apple-group-memberguid')
    memberuid = MultilineField(db_column='memberUid', max_length=200, blank=True, help_text='None', unique=False, verbose_name=u'Nom des membres')
    applegrouprealname = CharField(db_column='apple-group-realname', max_length=200, blank=True, help_text='group real name', unique=False, verbose_name=u'Nom du groupe')
    description = CharField(db_column='description', max_length=200, blank=True, help_text='RFC4519: descriptive information', unique=False, verbose_name=u'description')

    class Meta:
        verbose_name = u"""Groupe d'ordinateur"""
        verbose_name_plural = u"""Groupes d'ordinateurs"""

    def save(self, *args, **kwargs):
        if not self.applegenerateduid:
            self.applegenerateduid = unicode(uuid.uuid1())
            computer_list = ComputerList(cn=self.cn, applegenerateduid=self.applegenerateduid, applecomputers=self.memberuid)
            computer_list.save()
        else:
            computer_lists = ComputerList.objects.filter(applegenerateduid=self.applegenerateduid)[0:1]
            for computer_list in computer_lists:
                computer_list.cn = self.cn
                computer_list.applecomputers = self.memberuid
                computer_list.save()
        if not self.gidnumber:
            l = ComputerGroups.objects.order_by('-gidnumber')[0:1]
            if len(l) == 1:
                self.gidnumber = l[0].gidnumber + 1
            else:
                self.gidnumber = OPENDIRADMIN_FIRST_GID_NUMBER + 1
        super(ComputerGroups, self).save(*args, **kwargs)

    def __str__(self):
        return self.cn

    def __unicode__(self):
        return self.cn


class Mounts(ldapdb.models.Model):
    u"""
            
        """    
    base_dn = 'cn=mounts,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'mount']
    cn = CharField(primary_key=True, db_column='cn', max_length=200, blank=False, help_text='RFC4519: common name(s) for which the entity is known by', unique=True, verbose_name='cn')
    mountdirectory = CharField(primary_key=False, db_column='mountDirectory', max_length=200, blank=True, help_text='mount path', unique=False, verbose_name='mountDirectory')
    mounttype = CharField(primary_key=False, db_column='mountType', max_length=200, blank=True, help_text='mount VFS type', unique=False, verbose_name='mountType')
    mountoption = CharField(primary_key=False, db_column='mountOption', max_length=200, blank=True, help_text='mount options', unique=False, verbose_name='mountOption')
#mountType (nfs, url), mountOption=(net, url==afp://;AUTH=NO%20USER%20AUTHENT@central.19pouces.net/PARTAGES), mountDirectory
    class Meta:
        verbose_name = u"""Point de montage"""
        verbose_name_plural = u"""Points de montage"""

    def __str__(self):
        return self.cn

    def __unicode__(self):
        return self.cn


class ComputerList(ldapdb.models.Model):
    u"""
            
        """    
    base_dn = 'cn=computer_lists,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'apple-computer-list']
    cn = CharField(primary_key=True, db_column='cn', max_length=200, blank=False, help_text='RFC4519: common name(s) for which the entity is known by', unique=True, verbose_name='cn')
    applegenerateduid = CharField(primary_key=False, db_column='apple-generateduid', max_length=200, blank=True, help_text='generated unique ID', unique=False, verbose_name='apple-generateduid')
    applecomputers = CharField(primary_key=False, db_column='apple-computers', max_length=200, blank=True, help_text='computers', unique=False, verbose_name='apple-computers')
    class Meta:
        verbose_name = u"""Liste de machines"""
        verbose_name_plural = u"""Listes de machines"""

    def __str__(self):
        return self.cn

    def __unicode__(self):
        return self.cn


class Service(ldapdb.models.Model):
    u"""
            
        """    
    base_dn = 'cn=services,' + OPENDIRADMIN_BASE
    object_classes = ['top', 'apple-service', ]
    cn = CharField(primary_key=True, db_column='cn', max_length=200, blank=False, help_text='RFC4519: common name(s) for which the entity is known by', unique=True, verbose_name='Nom commun')
    appleservicetype = CharField(primary_key=False, db_column='apple-service-type', max_length=200, blank=False, help_text='type of service', unique=False, verbose_name='Type de service')
    iphostnumber = MultilineField(db_column='ipHostNumber', max_length=200, blank=True, unique=False, verbose_name=u'Adresses IP', help_text=u'Une valeur par ligne')
    applekeyword = CharField(primary_key=False, db_column='apple-keyword', max_length=200, blank=True, help_text='keywords', unique=False, verbose_name='apple-keyword')
    appleserviceurl = CharField(primary_key=False, db_column='apple-service-url', max_length=200, blank=True, help_text='URL of service', unique=False, verbose_name=u'URL complète du service')
    appleserviceport = IntegerField(primary_key=False, db_column='apple-service-port', blank=True, help_text='Service port number', unique=False, verbose_name='Port du service')
    applednsname = MultilineField(primary_key=False, db_column='apple-dnsname', max_length=200, blank=True, unique=False, verbose_name='Nom DNS', help_text=u'Une valeur par ligne')
    description = CharField(primary_key=False, db_column='description', max_length=200, blank=True, help_text='RFC4519: descriptive information', unique=False, verbose_name='description')
    #appleservicelocation = CharField(primary_key=False, db_column='apple-service-location', max_length=200, blank=True, help_text='Service location', unique=False, verbose_name='apple-service-location')
    #appleservicelocation = ForeignKey(Domains, to_field='name', db_column='apple-service-location', null=True, blank=True, verbose_name=u'Zone DNS')

    regen_certificate = models.BooleanField(u'Regénérer le certificat', max_length=20, default=BOOLEAN_CHOICES[0][0], choices=BOOLEAN_CHOICES, blank=True)
    regen_key = models.BooleanField(u'Regénérer la clef privée', max_length=20, default=BOOLEAN_CHOICES[0][0], choices=BOOLEAN_CHOICES, blank=True)
    regen_keytab = models.BooleanField(u'Regénérer la clef Kerberos', max_length=20, default=BOOLEAN_CHOICES[0][0], choices=BOOLEAN_CHOICES, blank=True)

    class Meta:
        verbose_name = u"""Service"""
        verbose_name_plural = u"""Services"""

    def __init__(self, *args, **kwargs):
        ldapdb.models.Model.__init__(self, *args, **kwargs)
        self.o__applednsname = self.applednsname
        self.o__iphostnumber = self.iphostnumber

    def __str__(self):
        return self.cn

    def __unicode__(self):
        return self.cn

