#coding=utf8
import codecs
from optparse import make_option
import re
import sys
import unicodedata

import ldap
import ldap.schema.subentry
from ldapdb.models.fields import CharField, IntegerField, ListField, ImageField
from django.core.management.base import BaseCommand, CommandError

from coreadmin.fields import MultilineField, DateTimeField, FloatField, BooleanField, OctetStringField
from OpenDirAdmin.settings import OPENDIRADMIN_BASE, OPENDIRADMIN_URL

def slugify(value):
    """
    Normalizes string, converts to lowercase, removes non-alpha characters,
    and converts spaces to hyphens.
    """
    value = unicodedata.normalize('NFKD', unicode(value)).encode('ascii', 'ignore')
    value = unicode(re.sub('[^\w\s_]', '', value).strip().lower())
    return re.sub('[_\s]+', '_', value)
LDAP_TYPES = {
        '1.3.6.1.1.16.1': ('CharField', {'max_length': 200}), #UUID 
        '1.3.6.1.4.1.1466.115.121.1.3': ('CharField', {'max_length': 200}),# Attribute Type Description
        '1.3.6.1.4.1.1466.115.121.1.5': ('CharField', {'max_length': 200}), # Binary syntax
        '1.3.6.1.4.1.1466.115.121.1.6': ('CharField', {'max_length': 200}),# Bit string syntax
        '1.3.6.1.4.1.1466.115.121.1.7': ('BooleanField', {}),# Boolean syntax
        '1.3.6.1.4.1.1466.115.121.1.8': ('OctetStringField', {'max_length': 200}),# Certificate syntax
        '1.3.6.1.4.1.1466.115.121.1.9': ('OctetStringField', {'max_length': 200}), # Certificate List syntax
        '1.3.6.1.4.1.1466.115.121.1.10': ('OctetStringField', {'max_length': 200}),# Certificate Pair syntax
        '1.3.6.1.4.1.1466.115.121.1.11': ('CharField', {'max_length': 200}), # Country String syntax
        '1.3.6.1.4.1.1466.115.121.1.12': ('CharField', {'max_length': 200}), # Distinguished Name syntax
        '1.3.6.1.4.1.1466.115.121.1.13': ('CharField', {'max_length': 200}), # Data Quality Syntax
        '1.3.6.1.4.1.1466.115.121.1.14': ('CharField', {'max_length': 200}), # Delivery Method
        '1.3.6.1.4.1.1466.115.121.1.15': ('CharField', {'max_length': 200}), # Directory String syntax
        '1.3.6.1.4.1.1466.115.121.1.16': ('CharField', {'max_length': 200}), # DIT Content Rule syntax
        '1.3.6.1.4.1.1466.115.121.1.17': ('CharField', {'max_length': 200}), # DIT Structure Rule Description syntax
        '1.3.6.1.4.1.1466.115.121.1.19': ('CharField', {'max_length': 200}), # DSA Quality Syntax
        '1.3.6.1.4.1.1466.115.121.1.21': ('CharField', {'max_length': 200}), # Enhanced Guide
        '1.3.6.1.4.1.1466.115.121.1.22': ('CharField', {'max_length': 200}), # Facsimile Telephone Number syntax
        '1.3.6.1.4.1.1466.115.121.1.23': ('CharField', {'max_length': 200}), # Fax image syntax
        '1.3.6.1.4.1.1466.115.121.1.24': ('DateTimeField', {}), # Generalized Time syntax
        '1.3.6.1.4.1.1466.115.121.1.26': ('CharField', {'max_length': 200}), # IA5 String syntax
        '1.3.6.1.4.1.1466.115.121.1.27': ('IntegerField', {}), # Integer syntax
        '1.3.6.1.4.1.1466.115.121.1.28': ('OctetStringField', {'max_length': 200}), # JPeg Image syntax
        '1.3.6.1.4.1.1466.115.121.1.30': ('CharField', {'max_length': 200}), # Matching Rule Description syntax
        '1.3.6.1.4.1.1466.115.121.1.31': ('CharField', {'max_length': 200}), # Matching Rule Use Description syntax
        '1.3.6.1.4.1.1466.115.121.1.33': ('CharField', {'max_length': 200}), # MHS OR Address syntax
        '1.3.6.1.4.1.1466.115.121.1.34': ('CharField', {'max_length': 200}), # Name and Optional UID syntax
        '1.3.6.1.4.1.1466.115.121.1.35': ('CharField', {'max_length': 200}), # Name Form syntax
        '1.3.6.1.4.1.1466.115.121.1.36': ('FloatField', {}), # Numeric String syntax
        '1.3.6.1.4.1.1466.115.121.1.37': ('CharField', {'max_length': 200}), # Object Class Description syntax
        '1.3.6.1.4.1.1466.115.121.1.38': ('CharField', {'max_length': 200}), # OID syntax
        '1.3.6.1.4.1.1466.115.121.1.39': ('CharField', {'max_length': 200}), # Other Mailbox syntax
        '1.3.6.1.4.1.1466.115.121.1.40': ('CharField', {'max_length': 200}), # Octet String
        '1.3.6.1.4.1.1466.115.121.1.41': ('CharField', {'max_length': 200}), # Postal Address syntax
        '1.3.6.1.4.1.1466.115.121.1.43': ('CharField', {'max_length': 200}), # Presentation Address syntax
        '1.3.6.1.4.1.1466.115.121.1.44': ('CharField', {'max_length': 200}), # Printable string syntax
        '1.3.6.1.4.1.1466.115.121.1.49': ('CharField', {'max_length': 200}), # Supported Algorithm
        '1.3.6.1.4.1.1466.115.121.1.50': ('CharField', {'max_length': 200}), # Telephone number syntax
        '1.3.6.1.4.1.1466.115.121.1.51': ('CharField', {'max_length': 200}), # Teletex Terminal Identifier
        '1.3.6.1.4.1.1466.115.121.1.52': ('CharField', {'max_length': 200}), # Telex Number
        '1.3.6.1.4.1.1466.115.121.1.53': ('CharField', {'max_length': 200}), # UTCTime syntax
        '1.3.6.1.4.1.1466.115.121.1.54': ('CharField', {'max_length': 200}), # LDAP Syntax Description syntax
        }

OD_MAPPINGS = [
        {'class_name': 'Container', 'dn': "", 'primary_key': 'cn', 'object_classes': ['top', 'container', ], 'verbose_name': u'Conteneur', 'verbose_name_plural': u'Conteneurs', },
        {'class_name': 'CertificateAuthorities', 'dn': "cn=certificateauthorities", 'primary_key': 'ou', 'object_classes': ['top', 'pkiCA', 'organizationalUnit', ],
'verbose_name': u'Autorité de certification', 'verbose_name_plural': u'Autorités de certification', },
        {'class_name': 'ComputerGroups', 'dn': "cn=computer_groups", 'primary_key': 'cn', 'object_classes': ['top', 'posixGroup', 'extensibleObject', 'apple-group', ],
'verbose_name': u'Groupe d\'ordinateur', 'verbose_name_plural': u'Groupes d\'ordinateurs', },
        {'class_name': 'Computer', 'dn': "cn=computers", 'primary_key': 'cn', 'object_classes': ['top', 'apple-computer', ],
'verbose_name': u'Ordinateur', 'verbose_name_plural': u'Ordinateurs', },
        {'class_name': 'Config', 'dn': "cn=config", 'primary_key': 'cn', 'object_classes': ['top', 'apple-configuration', ],
'verbose_name': u'Configuration', 'verbose_name_plural': u'Configurations', },
        {'class_name': 'Groups', 'dn': "cn=groups", 'primary_key': 'cn', 'object_classes': ['top', 'posixGroup', 'extensibleObject', 'apple-group', ],
'verbose_name': u'Groupe', 'verbose_name_plural': u'Groupes', },
        {'class_name': 'Users', 'dn': "cn=users", 'primary_key': 'uid', 'object_classes': ['top', 'person', 'inetOrgPerson', 'organizationalPerson', 'posixAccount', 'shadowAccount', 'apple-user', 'extensibleObject', ],
'verbose_name': u'Utilisateur', 'verbose_name_plural': u'Utilisateurs', },
        {'class_name': 'PresetComputerGroups', 'dn': "cn=presets_computer_groups", 'primary_key': 'cn', 'object_classes': ['top', 'posixGroup', 'extensibleObject', 'apple-group', ],
'verbose_name': u'Préréglage de groupe d\'ordinateur', 'verbose_name_plural': u'Préréglages de groupes d\'ordinateurs', },
        {'class_name': 'PresetComputers', 'dn': "cn=presets_computers", 'primary_key': 'cn', 'object_classes': ['top', 'apple-computer', ],
'verbose_name': u'Préréglage d\'ordinateur', 'verbose_name_plural': u'Préréglages d\'ordinateurs', },
        {'class_name': 'PresetGroups', 'dn': "cn=presets_groups", 'primary_key': 'cn', 'object_classes': ['top', 'posixGroup', 'extensibleObject', 'apple-group', ],
'verbose_name': u'Préréglage de groupe', 'verbose_name_plural': u'Préréglages de groupes', },
        {'class_name': 'Mounts', 'dn': "cn=mounts", 'primary_key': 'cn', 'object_classes': ['top', 'mount', ],
'verbose_name': u'Point de montage', 'verbose_name_plural': u'Points de montage', },
#mountType (nfs, url), mountOption=(net, url==afp://;AUTH=NO%20USER%20AUTHENT@central.19pouces.net/PARTAGES), mountDirectory
        {'class_name': 'ComputerList', 'dn': "cn=computer_lists", 'primary_key': 'cn', 'object_classes': ['top', 'apple-computer-list', ],
'verbose_name': u'Liste de machines', 'verbose_name_plural': u'Listes de machines', },
        {'class_name': 'CertificateRequest', 'dn': "cn=certificaterequests", 'primary_key': 'apple-transactionID', 'object_classes': ['top', 'apple-certificateRequestInfo', ],
'verbose_name': u'Requête de certificat', 'verbose_name_plural': u'Requêtes de certificat', },
        {'class_name': 'PresetUsers', 'dn': "cn=presets_users", 'primary_key': 'uid', 'object_classes': ['top', 'person', 'inetOrgPerson', 'organizationalPerson', 'posixAccount', 'shadowAccount', 'apple-user', 'extensibleObject', ],
'verbose_name': u'Préréglage d\'utilisateur', 'verbose_name_plural': u'Préréglages d\'utilisateurs', }, ]
OD_MAPPINGS = [
        {'class_name': 'Service', 'dn': "cn=resources", 'primary_key': 'cn', 'object_classes': ['top', 'apple-service', 'apple-computer-info', ],
'verbose_name': u'Service', 'verbose_name_plural': u'Services', },
]
OD_INITIAL_DATA = ('users', 'groups', 'mounts', 'accesscontrols', 'certificateauthorities', 'certificaterequests', 'computers',
        'computer_groups', 'computer_lists', 'config', 'locations', 'machines', 'neighborhoods', 'people', 'presets_computer_lists',
        'presets_groups', 'presets_users', 'printers', 'augments', 'autoserversetup', 'filemakerservers', 'resources', 'places',
        'maps', 'presets_computers', 'presets_computer_groups', 'automountMap',)

class Command(BaseCommand):
    args = '<ClassName> <dn> <primary_key>'
    help = u'Write class '
    option_list = BaseCommand.option_list + (
        make_option(u'--class', action='append', dest='object_classes', default=['top'], help=u''),
        make_option(u'--verbose_name', action='store', dest='verbose_name', default=None, help=u''),
        make_option(u'--verbose_name_plural', action='store', dest='verbose_name_plural', default=None, help=u''),
        )

    def handle(self, *args, **options):
        if len(args) == 3:
            mappings = ({'class_name': args[0], 'dn': args[1], 'primary_key': args[2], 'object_classes': options['object_classes'], 'verbose_name': options['verbose_name'],
                    'verbose_name_plural': options['verbose_name_plural']},)
        else:
            mappings = OD_MAPPINGS

            self.stdout.write(u"""#coding=utf8
from django.db import models
from django.db.models import fields, SubfieldBase

import ldapdb.models
from ldapdb.models.fields import CharField, IntegerField, ListField, ImageField

from coreadmin.fields import MultilineField, DateTimeField, FloatField, BooleanField, OctetStringField
from OpenDirAdmin.settings import OPENDIRADMIN_BASE

""".encode('utf-8'))
        for mapping in mappings:
            primary_key = mapping['primary_key']
            schema = ldap.schema.subentry.urlfetch(OPENDIRADMIN_URL)
            must, may = schema[1].attribute_types(mapping['object_classes'])
            fields = ''
            fields_dict = {}
            for d, blank in (must, False), (may, True):
                for key, attribute in d.items():
                    name = attribute.names[0]
                    if name[0:3] == 'olc' or name == 'objectClass': continue
                    args = {'db_column': "'" + name + "'", 'unique': name == primary_key, 'blank': blank, 'primary_key': name == primary_key, 'verbose_name': "'" + name + "'", 'help_text': "'" + unicode(attribute.desc) +"'"}
                    if attribute.syntax in LDAP_TYPES:
                        field_type = LDAP_TYPES[attribute.syntax][0]
                        args.update(LDAP_TYPES[attribute.syntax][1])
                    elif attribute.single_value or name == primary_key:
                        self.stderr.write(name + ': ' + unicode(attribute.syntax) + "\n")
                        field_type = 'CharField'
                        args.update({'max_length': 200})
                    else:
                        self.stderr.write(name + ': ' + unicode(attribute.syntax) + "\n")
                        field_type = 'MultilineField'
                        args.update({'max_length': 200})
                    field_str = '    {0} = {1}({2})'.format(slugify(name), field_type, ', '.join([x + '=' + unicode(y) for (x, y) in args.items()]))
                    fields_dict[key] = field_str
            keys = fields_dict.keys()
            keys.sort()
            fields = "\n".join([fields_dict[key] for key in keys])
            meta = ''
            verbose_name = mapping['verbose_name']
            verbose_name_plural = mapping['verbose_name_plural']
            if verbose_name or verbose_name_plural:
                meta = u"    class Meta:\n"
            if verbose_name:
                meta += u"        verbose_name = u\"\"\"{0}\"\"\"\n".format(verbose_name)
            if verbose_name_plural:
                meta += u"        verbose_name_plural = u\"\"\"{0}\"\"\"\n".format(verbose_name_plural)
            values = { 'object_classes_str': "', '".join(mapping['object_classes']), 'fields': fields, 'meta': meta, }
            values.update(mapping)
            values['base_dn'] = values['dn'] and values['dn'] + ',' or ''
            self.stdout.write(u"""

class {class_name}(ldapdb.models.Model):
    u\"\"\"
            
        \"\"\"    
    base_dn = '{base_dn}' + OPENDIRADMIN_BASE
    object_classes = ['{object_classes_str}']
{fields}
{meta}
    def __str__(self):
        return self.{primary_key}

    def __unicode__(self):
        return self.{primary_key}

""".format(**values).encode('utf-8'))



